import shutil
import os
import requests
import tarfile
import zipfile
import json


JAVA_URL = "https://download.java.net/java/GA/jdk16.0.2/d4a915d82b4c4fbb9bde534da945d746/7/GPL/openjdk-16.0.2_linux-x64_bin.tar.gz"
GML_TOOLS_URL = "https://github.com/citygml4j/citygml-tools/releases/download/v1.4.2/citygml-tools-1.4.2.zip"
LOCAL_JAVA_FILE = "java.tar.gz"
LOCAL_GML_TOOLS_FILE = "gmltools.zip"

def create_dir_if_not_exists(dir):
    if not os.path.exists(dir):
        os.makedirs(dir)


def recreate_dir(dir):
    if os.path.exists(dir):
        shutil.rmtree(dir)
    create_dir_if_not_exists(dir)


def get_tools_dir(workspace_path):
    tools_dir = os.path.join(workspace_path, "tools")
    create_dir_if_not_exists(tools_dir)
    return tools_dir


def get_tools_dir_module(workspace_path, module):
    tools_module_dir = os.path.join(workspace_path, "tools", module)
    create_dir_if_not_exists(tools_module_dir)
    return tools_module_dir


def update_config(config_file, key, value):
    with open(config_file, "w+") as config:
        try:
            data = json.load(config)
        except:
            data = {}
        data[key] = value
        json.dump(data, config)


def download(url, dst):
    data = requests.get(url)
    with open(dst, 'wb') as file:
        file.write(data.content)


def download_java(tools_dir):
    url = JAVA_URL
    dst = os.path.join(tools_dir, LOCAL_JAVA_FILE)
    download(url, dst)
    return dst


def download_gml_tools(tools_dir):
    url = GML_TOOLS_URL
    dst = os.path.join(tools_dir, LOCAL_GML_TOOLS_FILE)
    download(url, dst)
    return dst


def untar(src, dst):
    with tarfile.open(src) as file:
        file.extractall(dst)
    os.remove(src)
    files = os.listdir(dst)
    if len(files) != 1:
        raise Exception(f"Unexpected list of files in extracted directory: {files}")
    return files[0]


def unzip(src, dst):
    with zipfile.ZipFile(src, 'r') as file:
        file.extractall(dst)
    os.remove(src)
    files = os.listdir(dst)
    if len(files) != 1:
        raise Exception(f"Unexpected list of files in extracted directory: {files}")
    return files[0]


def install_java(workspace_path: str):
    tools_dir = get_tools_dir_module(workspace_path, "java")    
    java_archive = download_java(tools_dir)
    java_dir = untar(java_archive, tools_dir)
    return os.path.abspath(os.path.join(tools_dir, java_dir))


def install_gml_tools(workspace_path: str):
    tools_dir = get_tools_dir_module(workspace_path, "gmltools")  
    gml_tools_archive = download_gml_tools(tools_dir)
    gml_tools_dir = unzip(gml_tools_archive, tools_dir)
    return os.path.abspath(os.path.join(tools_dir, gml_tools_dir))


def reinstall(workspace_path: str):
    config = {}
    print(f"Creating workspace directory...")
    recreate_dir(workspace_path)
    print(f"Installing Java...")
    config["java"] = install_java(workspace_path)
    print(f"Installing GML Tools...")
    config["gmltools"] = install_gml_tools(workspace_path)
    return config



import json
import os
from metaworkspace import init, install


class MetacityWorkspace:
    def __init__(self, workspace_path: str):
        self.path = os.path.abspath(workspace_path)


    @property
    def config_file(self):
        return os.path.join(self.path, "config.json")


    def update_config(self, key, value):
        with open(self.config_file, "w+") as config:
            try:
                data = json.load(config)
            except:
                data = {}
            data[key] = value
            json.dump(data, config)


    @property
    def config(self):
        with open(self.config_file, "r") as file:
            return json.load(file)


    def init(self):
        config = self.config
        init.setup_java(config)


    def reinstall(self):
        config = install.reinstall(self.path)
        for k, v in config.items():
            self.update_config(k, v)


from metaworkspace.workspace import MetacityWorkspace
from argparse import ArgumentParser


usage = ("Sets up Metacity Workspace."
         "Downloads all Metacity modules, sets up local installation of Java,"
         "GMLTools")


if __name__ == "__main__":
    parser = ArgumentParser(description=usage)
    parser.add_argument('workspace_dir', type=str, help='Path to newly created Metacity workspace')
    args = parser.parse_args()    
    path = args.workspace_dir
    mws = MetacityWorkspace(path)
    mws.reinstall() 


import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="metaworkspace",
    packages=find_packages("."),
    version="0.0.1",
    description="Python tool simplyfing work with Metacity Modules",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/MetacitySuite/Metacity-workspace",
    author="Metacity",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ],
    python_requires='>=3.8',                # Minimum version requirement of the package
    install_requires=[
        "metacity>=0.0.11",
        "requests>=2.26.0"
    ] # Install other dependencies if any
)